@@ -171,37 +171,92 @@ func (s *ligoloServer) StopRelay(ctx context.Context, in *pb.StopRelayReq) (*pb.
 	return &pb.Empty{}, err
 }
 
-func (s *ligoloServer) AddRoute(ctx context.Context, in *pb.AddRouteReq) (*pb.AddRouteResp, error) {
+func (s *ligoloServer) AddRoute(ctx context.Context, in *pb.AddRouteReq) (*pb.Empty, error) {
 	slog.Debug("Received request to create route", slog.Any("in", in))
 
-	if !in.Force {
-		if overlappingSession, overlappingRoute := s.sessService.RouteOverlaps(in.Route.Cidr); overlappingSession != nil {
-			return nil, fmt.Errorf("route overlaps with route %s in session %s", overlappingRoute, overlappingSession.GetName())
-		}
-	}
-
 	sess := s.sessService.GetSession(in.SessionID)
-	err := s.sessService.NewRoute(in.SessionID, in.Route.Cidr, in.Route.IsLoopback)
-	if err == nil {
-		oper := ctx.Value("operator").(*operator.Operator)
-		if in.Route.IsLoopback {
-			events.Publish(events.OK, "%s: loopback route '%s' added to '%s'", oper.Name, in.Route.Cidr, sess.GetName())
-		} else {
-			events.Publish(events.OK, "%s: regular route '%s' added to '%s'", oper.Name, in.Route.Cidr, sess.GetName())
-		}
+	err := s.sessService.NewRoute(in.SessionID, in.Route.Cidr, int(in.Route.Metric), in.Route.IsLoopback)
+	if err != nil {
+		return nil, err
 	}
 
-	return &pb.AddRouteResp{}, err
+	routeType := "regular"
+	if in.Route.IsLoopback {
+		routeType = "loopback"
+	}
+
+	oper := ctx.Value("operator").(*operator.Operator)
+	events.Publish(events.OK, "%s: %s route '%s' with metric '%d' added to '%s'", oper.Name, routeType, in.Route.Cidr, in.Route.Metric, sess.GetName())
+
+	return &pb.Empty{}, nil
+}
+
+func (s *ligoloServer) EditRoute(ctx context.Context, in *pb.EditRouteReq) (*pb.Empty, error) {
+	slog.Debug("Received request to edit route", slog.Any("in", in))
+
+	oldRoute, err := s.sessService.RemoveRoute(in.SessionID, in.RouteID)
+	if err != nil {
+		return &pb.Empty{}, err
+	}
+
+	err = s.sessService.NewRoute(in.SessionID, in.Route.Cidr, int(in.Route.Metric), in.Route.IsLoopback)
+	if err != nil {
+		s.sessService.NewRoute(in.SessionID, oldRoute.Cidr.String(), int(oldRoute.Metric), oldRoute.IsLoopback)
+		return &pb.Empty{}, err
+	}
+
+	oldRouteType := "regular"
+	if oldRoute.IsLoopback {
+		oldRouteType = "loopback"
+	}
+
+	newRouteType := "regular"
+	if in.Route.IsLoopback {
+		newRouteType = "loopback"
+	}
+
+	oper := ctx.Value("operator").(*operator.Operator)
+	sess := s.sessService.GetSession(in.SessionID)
+	events.Publish(events.OK, "%s: %s route '%s' on '%s' changed to %s route '%s' with metric '%d'", oper.Name, oldRouteType, oldRoute.Cidr.String(), sess.GetName(), newRouteType, in.Route.Cidr, in.Route.Metric)
+
+	return &pb.Empty{}, nil
+}
+
+func (s *ligoloServer) MoveRoute(ctx context.Context, in *pb.MoveRouteReq) (*pb.Empty, error) {
+	slog.Debug("Received request to move route", slog.Any("in", in))
+
+	oldRoute, err := s.sessService.RemoveRoute(in.OldSessionID, in.RouteID)
+	if err != nil {
+		return &pb.Empty{}, err
+	}
+
+	err = s.sessService.NewRoute(in.NewSessionID, oldRoute.Cidr.String(), oldRoute.Metric, oldRoute.IsLoopback)
+	if err != nil {
+		s.sessService.NewRoute(in.OldSessionID, oldRoute.Cidr.String(), oldRoute.Metric, oldRoute.IsLoopback)
+		return &pb.Empty{}, err
+	}
+
+	routeType := "regular"
+	if oldRoute.IsLoopback {
+		routeType = "loopback"
+	}
+
+	oper := ctx.Value("operator").(*operator.Operator)
+	oldSess := s.sessService.GetSession(in.OldSessionID)
+	newSess := s.sessService.GetSession(in.NewSessionID)
+	events.Publish(events.OK, "%s: %s route '%s' on '%s' moved to '%s'", oper.Name, routeType, oldRoute.Cidr.String(), oldSess.GetName(), newSess.GetName())
+
+	return &pb.Empty{}, nil
 }
 
 func (s *ligoloServer) DelRoute(ctx context.Context, in *pb.DelRouteReq) (*pb.Empty, error) {
 	slog.Debug("Received request to delete route", slog.Any("in", in))
 
 	sess := s.sessService.GetSession(in.SessionID)
-	err := s.sessService.RemoveRoute(in.SessionID, in.Cidr)
+	route, err := s.sessService.RemoveRoute(in.SessionID, in.RouteID)
 	if err == nil {
 		oper := ctx.Value("operator").(*operator.Operator)
-		events.Publish(events.OK, "%s: route '%s' removed from '%s'", oper.Name, in.Cidr, sess.GetName())
+		events.Publish(events.OK, "%s: route '%s' removed from '%s'", oper.Name, route.Cidr, sess.GetName())
 	}
 
 	return &pb.Empty{}, err
@@ -263,6 +318,24 @@ func (s *ligoloServer) GenerateAgent(ctx context.Context, in *pb.GenerateAgentRe
 	return &pb.GenerateAgentResp{AgentBinary: result}, nil
 }
 
+func (s *ligoloServer) Traceroute(ctx context.Context, in *pb.TracerouteReq) (*pb.TracerouteResp, error) {
+	slog.Debug("Received request to trace address", slog.Any("in", in))
+
+	trace, err := s.sessService.Traceroute(in.IP)
+	if err != nil {
+		return nil, err
+	}
+
+	var protoTrace []*pb.Traceroute
+	for _, traceLine := range trace {
+		protoTrace = append(protoTrace, traceLine.Proto())
+	}
+
+	return &pb.TracerouteResp{
+		Trace: protoTrace,
+	}, nil
+}
+
 func (s *ligoloServer) GetOperators(ctx context.Context, in *pb.Empty) (*pb.GetOperatorsResp, error) {
 	slog.Debug("Received request to list operators", slog.Any("in", in))
 	oper := ctx.Value("operator").(*operator.Operator)
