@@ -38,10 +38,11 @@ func Remove(ID int) error {
 	return nil
 }
 
-func AddRoute(ID int, CIDR *net.IPNet) error {
+func AddRoute(ID int, CIDR *net.IPNet, metric int) error {
 	route := &netlink.Route{
 		LinkIndex: ID,
 		Dst:       CIDR,
+		Priority:  metric,
 	}
 	if err := netlink.RouteAdd(route); err != nil {
 		return err
@@ -75,3 +76,12 @@ func GetName(ID int) (string, error) {
 	}
 	return link.Attrs().Name, nil
 }
+
+func GetRoute(address net.IP) ([]netlink.Route, error) {
+	routes, err := netlink.RouteGet(address)
+	if err != nil {
+		return nil, err
+	}
+
+	return routes, nil
+}
